"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isMaciej = void 0;
const js_md5_1 = __importDefault(require("js-md5"));
const querystring_1 = __importDefault(require("querystring"));
const node_fetch_1 = __importStar(require("node-fetch"));
const atob_1 = __importDefault(require("atob"));
const btoa_1 = __importDefault(require("btoa"));
const isMaciej_1 = __importDefault(require("./utils/isMaciej"));
exports.isMaciej = isMaciej_1.default;
class Wykop {
    config;
    user;
    constructor(config) {
        this.config = {
            appkey: 'aNd401dAPp',
            secret: '',
            host: 'a2.wykop.pl',
            userAgent: 'wykop-v2-js/dev',
            ...config,
        };
        this.user = {
            isLoggedIn: false,
            login: null,
            password: null,
            accountkey: null,
            userkey: null,
        };
    }
    requestConfig(type, { api, named, post }) {
        let url;
        url = `https://${this.config.host}/${type}/`;
        if (api) {
            url += `${api.join('/')}/`;
        }
        if (named) {
            url += `${Object.keys(named)
                .map((key) => `${key}/${named[key]}`)
                .join('/')}/`;
        }
        url += `appkey/${this.config.appkey}/`;
        if (this.user.isLoggedIn && this.user.userkey) {
            url += `userkey/${this.user.userkey}`;
        }
        const headers = new node_fetch_1.Headers({
            'User-Agent': this.config.userAgent,
        });
        if (post) {
            headers.append('Content-Type', 'application/x-www-form-urlencoded');
        }
        // if signing header required
        if (!(0, isMaciej_1.default)(this.config.appkey)) {
            let signedContent = this.config.secret + url;
            if (post) {
                signedContent += Object.values(post).join(',');
            }
            headers.append('apisign', (0, js_md5_1.default)(signedContent));
        }
        let body;
        if (post) {
            body = querystring_1.default.stringify(post);
        }
        const method = post ? 'POST' : 'GET';
        return {
            a: url,
            b: {
                body,
                method,
                headers,
            },
        };
    }
    async request(type, { api, named, post }) {
        const config = this.requestConfig(type, { api, named, post });
        return new Promise((resolve, reject) => (0, node_fetch_1.default)(config.a, config.b)
            .then((res) => res.json())
            .catch((err) => { throw err; })
            .then((res) => {
            if (!res.error) {
                resolve(res);
            }
            else {
                // TODO: relogin if userkey expired and retry
                reject(res.error);
            }
        }));
    }
    async login(data) {
        if (typeof data === 'object') {
            if ((data.login && data.accountkey)
                || (data.login && data.password && (0, isMaciej_1.default)(this.config.appkey))) {
                return this.loginCallback(data);
            }
        }
        // handling data from Login/Connect
        if (typeof data === 'string') {
            const d = JSON.parse((0, atob_1.default)(data));
            if (d.appkey !== this.config.appkey) {
                throw new Error('Connect data for wrong appkey');
            }
            if (d.sign !== (0, js_md5_1.default)(this.config.secret + d.appkey + d.login + d.token)) {
                throw new Error('Manipulated connect data');
            }
            return this.loginCallback({
                login: d.login,
                accountkey: d.token,
            });
        }
        if (typeof data === 'undefined') {
            if (!this.user.login)
                throw new Error('Expected user.login');
            if (this.user.accountkey) {
                return this.loginCallback({
                    login: this.user.login,
                    accountkey: this.user.accountkey,
                });
            }
            if (!this.user.accountkey && this.user.password) {
                if ((0, isMaciej_1.default)(this.config.appkey)) {
                    return this.loginCallback({
                        login: this.user.login,
                        password: this.user.password,
                    });
                }
            }
        }
        throw new Error('No parseable login data');
    }
    async loginCallback(data) {
        return this.request('login/index', {
            post: data,
        }).then((res) => {
            this.user.accountkey = data.accountkey;
            this.user.isLoggedIn = true;
            this.user.login = res.data.profile.login;
            this.user.password = data.password;
            this.user.userkey = res.data.userkey;
            return res.data.profile;
        });
    }
    generateConnectURL(redirect) {
        let url = `https://${this.config.host}/login/connect/appkey/${this.config.appkey}/`;
        if (redirect) {
            url += `redirect/${encodeURIComponent((0, btoa_1.default)(redirect))}/`;
            url += `secure/${(0, js_md5_1.default)(this.config.secret + redirect)}/`;
        }
        return url;
    }
}
module.exports = Wykop;
exports.default = Wykop;
